#include "mainwindow.h"
#include "ui_mainwindow.h"
#include<QtWidgets>
#include<QString>
#include<QLabel>
QString MainWindow::settingFileName = "./pluin_setting.ini";

MainWindow::MainWindow(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    setAttribute(Qt::WA_DeleteOnClose);

    // 插件配置文件
    setting = new QSettings(MainWindow::settingFileName, QSettings::IniFormat);
    this->restoreWinStatus();

    // 托盘初始化
    tray= new QSystemTrayIcon(this);    //初始化托盘对象tray
    tray->setIcon(QIcon(QPixmap(":/img/manage.png")));//设定托盘图标，引号内是自定义的png图片路径
    tray->setToolTip("插件管理器"); //提示文字
    QString title="管理器提示";
    QString text="插件管理器已经开始运行！";
    tray->show();//让托盘图标显示在系统托盘上
    //tray->showMessage(title,text,QSystemTrayIcon::Information,3000); //最后一个参数为提示时长，默认10000，即10s

    //创建菜单项动作
    QAction * minimizeAction = new QAction("最小化", this);
    connect(minimizeAction, SIGNAL(triggered()), this, SLOT(hide()));
    QAction * restoreAction = new QAction("显示所有插件", this);
    connect(restoreAction, SIGNAL(triggered()), this, SLOT(trayMenuRestoreAction()));
    QAction * quitAction = new QAction("退出", this);
    setContextMenuPolicy(Qt::ActionsContextMenu);
    connect(quitAction, SIGNAL(triggered()), this, SLOT(trayMenuCloseAction())); //关闭应用

    //创建托盘菜单
    trayMenu = new QMenu(this);
    trayMenu->addAction(minimizeAction);
    trayMenu->addAction(restoreAction);
    trayMenu->addSeparator();
    trayMenu->addAction(quitAction);
    tray->setContextMenu(trayMenu);

     connect(tray,SIGNAL(activated(QSystemTrayIcon::ActivationReason)),this,SLOT(iconActivated(QSystemTrayIcon::ActivationReason)));
     //加载图标
     showHaveInfo();
     setWindowFlags(Qt::Dialog);
     setWindowTitle(tr("UOS插件管理器"));
}

MainWindow::~MainWindow()
{
    for(auto m :manageWidget){
        delete m;
    }
    delete ui;
}
void MainWindow::releasePlugin()
{
    for(auto m :manageWidget){
        delete m;
    }
    delete ui;
}

// 托盘图标点击时间处理
void MainWindow::iconActivated(QSystemTrayIcon::ActivationReason ireason)
{
    switch (ireason)
    {
    case QSystemTrayIcon::Trigger:
        this->restoreWinStatus();
        this->showNormal();
        break;
    case QSystemTrayIcon::DoubleClick:
        this->restoreWinStatus();
        this->showNormal();
        break;
    case QSystemTrayIcon::MiddleClick:
        break;
    default:
        break;
    }
}

// 托盘图标恢复窗口菜单处理
void MainWindow::trayMenuRestoreAction()
{
    this->restoreWinStatus();
    this->showNormal();
}

// 托盘图标关闭窗口菜单处理
void MainWindow::trayMenuCloseAction()
{
    QSettings setting(MainWindow::settingFileName, QSettings::IniFormat);
    setting.setValue("running", 0);
    releasePlugin();
    qApp->quit();
}

void MainWindow::closeEvent(QCloseEvent *event)
{
    if(tray->isVisible())
    {
        this->saveWinStatus();// 保存窗口状态
        hide(); //隐藏窗口
        event->ignore(); //忽略事件
    }
}
void MainWindow::hideEvent(QHideEvent *event)
{
    if(tray->isVisible())
    {
        this->saveWinStatus(); // 保存窗口状态
        hide(); //隐藏窗口
        event->ignore(); //忽略事件
    }
}

void MainWindow::saveWinStatus()
{
    this->setting->setValue("pos_x", this->x());
    this->setting->setValue("pos_y", this->y());
}

void MainWindow::restoreWinStatus()
{
    this->move(this->setting->value("pos_x").toInt(), this->setting->value("pos_y").toInt());
}

void MainWindow::addPlugin(PlugInfo info)
{
    QPushButton *button = new QPushButton();
    button->setMaximumSize(100,100);
    button->setIconSize(button->rect().size());
    button->setIcon(QIcon(info.icon));
    button->show();
    QLabel *labelName = new QLabel(info.plugName.toLatin1());


//    labelName->setText(QStringLiteral(info.plugName.toLatin1().data()));
    ui->mainLayout->addWidget(button,0,Qt::AlignHCenter);
    ui->mainLayout->addWidget(labelName,0,Qt::AlignHCenter);
    connect(button,&QPushButton::clicked,this,[=](){
         QPluginLoader pluginLoader(info.name);
         QObject *plugin = pluginLoader.instance();
         CalInterface *inter = qobject_cast<CalInterface *>(plugin);
         QWidget *newWidget = inter->getWidget(info.workPath);
         manageWidget.push_back(newWidget);
         newWidget->show();
    });
    QPluginLoader pluginLoader(info.name);
    QObject *plugin = pluginLoader.instance();
    CalInterface *inter = qobject_cast<CalInterface *>(plugin);
    QList<QWidget*>empty =inter->loadAll(info.workPath);
    manageWidget.append(empty);
    for(auto m:manageWidget){
        m->show();
    }

}

void MainWindow::showHaveInfo()
{
    //将所有的插件与其对应的图标建立联系
    //使用button对其启用
    QDir dirName{QDir::currentPath()};
    dirName.cd("./plugins");
    dirName.setFilter(QDir::NoDotAndDotDot|QDir::AllEntries);//去掉.和..的inode
    QStringList fileNames = dirName.entryList();//获取dir目录下的所有条目
    for(auto file:fileNames){
        dirName.cd(file);
        QString path = dirName.path()+"/";
        QSettings *loadFile = new QSettings(path+"config.ini",QSettings::IniFormat);
        PlugInfo info;
        info.workPath = dirName.path();
        info.name = path+loadFile->value("name").toString();
        info.icon = path+loadFile->value("icon").toString();
        info.version = loadFile->value("version").toString();
        info.plugName = loadFile->value("plugName").toString();
        addPlugin(info);
        dirName.cd("../");
    }
    ui->mainLayout->addStretch();

}







