#include "clockwidget.h"
#include<QWidget>
#include<math.h>
#include<QResizeEvent>
#include<QGroupBox>
#include<QTimer>
#include<QTime>
#include<QPainter>
#include<QAction>
#include<QHBoxLayout>
#include<QPushButton>
#include<QMessageBox>
#include<QSettings>
#include<QDir>
clockwidget::clockwidget(QWidget *parent) : QWidget(parent)
{
    QTimer *timer = new QTimer(this);  //连接信号与槽
    //时钟刷新显示
    connect(timer, SIGNAL(timeout()), this, SLOT(update()));
    timer->start(1000);
    //设置窗体名称与大小
    resize(150,150);
    createMenu();
    setWindowFlag(Qt::FramelessWindowHint);
    setAttribute(Qt::WA_TranslucentBackground);
    move(500,500);
}

clockwidget::clockwidget(QSettings *m, QString workPath)
{
    this->workPath = workPath;
    loadFile = m;
    QTimer *timer = new QTimer(this);  //连接信号与槽
    connect(timer, SIGNAL(timeout()), this, SLOT(update()));
    timer->start(1000);
    //设置窗体名称与大小
    createMenu();
    setWindowFlag(Qt::FramelessWindowHint);
    setAttribute(Qt::WA_TranslucentBackground);
    //设置窗口位置
    int x = loadFile->value("x").toInt();
    int y = loadFile->value("y").toInt();
    this->resize(loadFile->value("size").toSize());
    this->move(x,y);
}
QString clockwidget::getRand()
{
    int max = 8;
    QString tmp = QString("0123456789ABCDEFGHIJKLMNOPQRSTUVWXZY");
    QString str = QString();
    QTime t = QTime::currentTime();
    qsrand(t.msec()+t.second()*1000);
    for(int i = 0;i<max;i++){
        int ir = qrand()%tmp.length();
        str[i] = tmp.at(ir);
    }
    return str;
}
clockwidget::~clockwidget()
{
    if(isSave){
        QDir filePath{this->workPath};
        if(!filePath.exists("cache")){
            filePath.mkdir("cache");
        }
        filePath.cd("./cache");
        QString path =this->workPath+"/cache/"+getRand()+".ini";

        QSettings m(path,QSettings::IniFormat);
        m.setValue("x",this->x());
        m.setValue("y",this->y());
        m.setValue("size",this->size());
    }

}



void clockwidget::paintEvent(QPaintEvent *event)
{
    static const QPoint hourHand[3] =
    {
        QPoint(5, 14),
        QPoint(-5, 14),
        QPoint(0, -50)
    };
    static const QPoint minuteHand[3] =
    {
        QPoint(5, 14),
        QPoint(-5,14),
        QPoint(0, -70)
    };
    static const QPoint secondHand[3] =
    {
        QPoint(5, 14),
        QPoint(-5, 14),
        QPoint(0, -80)
    };
    static const QPoint line_long[2]=
    {
        QPoint(0,100),
        QPoint(0,80)
    };
    static const QPoint line_short[2]={
        QPoint(0,100),
        QPoint(0,95)
    };
    //绘制的范围
    int side = qMin(width(), height());
    //获取当前的时间
    QTime time = QTime::currentTime();
    //声明用来绘图用的painter
    QPainter painter(this);
    //绘制的图像反锯齿
    painter.setRenderHint(QPainter::Antialiasing);
    //重新定位坐标起始点，把坐标原点放到窗体的中央
    painter.translate(width() / 2, height() / 2);
    //在表盘内添加文字（任意）
    //painter.drawText(-75,-120,100,170,Qt::AlignRight,tr("ROLEX"));
    //设定画布的边界
    painter.scale(side / 200.0, side / 200.0);
    //填充边线设为黑色，也可以设置为NoPen
    painter.setPen( Qt::black );
    //画刷颜色设定
    painter.setBrush(Qt::black);
    //保存painter的状态
    painter.save();
    //设置painter的旋转角度
    painter.rotate(30.0 * ((time.hour() + time.minute() / 60.0)));
    //填充时针的区域
    painter.drawConvexPolygon(hourHand, 3);
    painter.restore();
    painter.setPen(Qt::black);
    //12个个刻度循环
    for (int i = 0; i < 12; ++i)
    {
        painter.drawLine(line_long[0],line_long[1]);
        painter.rotate(30.0);
    }
    //绘制时针转角、刻度
    painter.setPen(Qt::black);
    painter.setBrush(Qt::black);
    painter.save();
    painter.rotate(6.0 * (time.minute() + time.second() / 60.0));
    painter.drawConvexPolygon(minuteHand, 3);
    painter.restore();
    painter.setPen(Qt::black);
    for (int j = 0; j < 60; ++j)
    {
        if ((j % 5) != 0)
        painter.drawLine(line_short[0],line_short[1]);
        painter.rotate(6.0);
    }

     //绘制分针转角、刻度
    painter.setPen(Qt::black);
    painter.setBrush(Qt::black);
    painter.save();
    painter.rotate(6.0 * (time.minute() + time.second() / 60.0));
    painter.drawConvexPolygon(minuteHand, 3);
    painter.restore();
    painter.setPen(Qt::gray);
    for (int j = 0; j < 60; ++j)
    {
        if ((j % 5) != 0)
        painter.drawLine(line_short[0],line_short[1]);
        painter.rotate(6.0);
    }
   //绘制秒针转角
    painter.setPen(Qt::red);
    painter.setBrush(Qt::red);
    painter.save();
    painter.rotate(6.0*time.second());
    painter.drawConvexPolygon(secondHand,3);
    painter.restore();
}

void clockwidget::createMenu()
{
    QAction *setSize = new QAction(tr("设置大小"));
    QAction *closeAction = new QAction("关闭");
    addAction(setSize);
    addAction(closeAction);
    connect(setSize,&QAction::triggered,this,&clockwidget::setSizeWindow);
    connect(closeAction,&QAction::triggered,this,[&](){
        isSave = false;
        close();
    });
    setContextMenuPolicy(Qt::ActionsContextMenu);
}

void clockwidget::setSizeWindow()
{
    if(sizeWindow == NULL){
        sizeWindow = new QWidget;
        QHBoxLayout *mainLayout = new QHBoxLayout;
        QPushButton *sizeOne = new QPushButton(tr("1X1"));
        QPushButton *sizeTwo = new QPushButton(tr("2X2"));
        QPushButton *sizeThree = new QPushButton(tr("3x3"));
        mainLayout->addWidget(sizeOne);
        mainLayout->addWidget(sizeTwo);
        mainLayout->addWidget(sizeThree);
        connect(sizeOne,&QPushButton::clicked,this,[&](){
            resize(150,150);
        });
        connect(sizeTwo,&QPushButton::clicked,this,[&](){
            resize(300,300);
        });
        connect(sizeThree,&QPushButton::clicked,this,[&](){
            resize(450,450);
        });
        sizeWindow->setLayout(mainLayout);
    }


    sizeWindow->move(this->x(),this->y());
    sizeWindow->show();

}

void clockwidget::setWorkPath(QString workPath)
{
    this->workPath = workPath;
}

void clockwidget::mousePressEvent(QMouseEvent *e)
{
    if(e->button()==Qt::LeftButton){
        moveIf = true;
        clickPos = e->pos();
    }
    return QWidget::mousePressEvent(e);
}

void clockwidget::mouseMoveEvent(QMouseEvent *e)
{
    if(moveIf)move(e->globalPos()-clickPos);
    return QWidget::mouseMoveEvent(e);
}

void clockwidget::mouseReleaseEvent(QMouseEvent *e)
{
    moveIf =false;
    return QWidget::mouseReleaseEvent(e);
}

