#include "weatherdataloader.h"

#include <QtNetwork/QNetworkRequest>
#include <QtNetwork/QNetworkReply>
#include <QJsonDocument>
#include <QJsonObject>
#include <QJsonArray>
#include <QList>
WeatherDataLoader::WeatherDataLoader()
{
    manager = new QNetworkAccessManager(this);
    connect(manager, SIGNAL(finished(QNetworkReply*)),this, SLOT(onResponsed(QNetworkReply*)));
//    connect(manager, SIGNAL(error(QNetworkReply::NetworkError)),this, SLOT(loadWeatherFailed(QNetworkReply::NetworkError)));
}

void WeatherDataLoader::queryCityWeather(QString city)
{
    // 1.第一次启动需要请求城市经纬度
    if (this->city_init == false) {
        this->requestCityName();
    } else {
        this->requestWeatherData();
    }
}

QString WeatherDataLoader::getCityName()
{
    if (this->city_init == true) {
        return this->city.name;
    } else {
        return "";
    }
}

void WeatherDataLoader::requestCityName()
{
    char quest_array[256]="http://ip-api.com/json?&&lang=zh-CN";
    QNetworkRequest quest;
    quest.setUrl(QUrl(quest_array));
    quest.setHeader(QNetworkRequest::UserAgentHeader,"RT-Thread ART");
    manager->get(quest);
//    return wd;
}

void WeatherDataLoader::requestWeatherData()
{
    QString url =QString("https://devapi.qweather.com/v7/weather/10d?location=%1,%2&key=%3").arg(this->city.lon).arg(this->city.lat).arg("a5104fe408e44f8f95a0c9d2eb9a5f65");
    QNetworkRequest quest;
    quest.setUrl(QUrl(url));
    quest.setHeader(QNetworkRequest::UserAgentHeader,"RT-Thread ART");
    manager->get(quest);
}

void WeatherDataLoader::onGetCityName(QJsonObject data)
{
    this->city.lat = data.value("lat").toDouble();
    this->city.lon = data.value("lon").toDouble();
    this->city.name = data.value("regionName").toString();
    this->city_init = true;
    this->requestWeatherData();
}

void WeatherDataLoader::onGetWeatherData(QJsonObject data)
{
    QJsonArray qja = data.value("daily").toArray();
    QList <WeatherData> wdList;
    // 解析天气数据
    for (int i = 0; i < qja.size(); i++) {
        WeatherData wd;
        QJsonObject today_weather = qja.at(i).toObject();
        wd.date = today_weather.value("fxDate").toString();
        wd.type = today_weather.value("textDay").toString();

        QString low = today_weather.value("tempMin").toString();
        QString high = today_weather.value("tempMax").toString();
        wd.degree = low.mid(low.length()-3,4) +"~"+ high.mid(high.length()-3,4);
        QString strength = today_weather.value("windScaleDay").toString();
        wd.windy = today_weather.value("windDirDay").toString() + strength;
        wd.icon = today_weather.value("iconDay").toString();
        wdList.append(wd);
    }
    QVariant qvData;
    qvData.setValue(wdList);
    emit this->succeed(qvData);
}

void WeatherDataLoader::onResponsed(QNetworkReply* reply)
{
    if (reply == nullptr) {
        emit this->failed(-1);
        return ;
    }
    qDebug()<<"recv weather data!!";
    QString all = reply->readAll();
    qDebug() << all; //将接收到的数据显示出来

    QJsonParseError err;
    QJsonDocument json_recv = QJsonDocument::fromJson(all.toUtf8(),&err);//解析json对象
    qDebug() << err.error;
    qDebug() << json_recv;

    if(!json_recv.isNull()) {
        QJsonObject object = json_recv.object();
        if (object.contains("city")) {
            this->onGetCityName(object);
        } else if (object.contains("fxLink")) {
            this->onGetWeatherData(object);
        } else {
            qDebug()<<"json_recv is empty object !!";
            emit this->failed(-1);
            return ;
        }

    } else {
        qDebug()<<"json_recv is NULL or is not a object !!";
        emit this->failed(-1);
        return ;
    }
    reply->deleteLater(); //销毁请求对象
}

void WeatherDataLoader::onFailed(QNetworkReply::NetworkError error)
{
//    emit this->failed(-1);
}

