#include "weatherwidget.h"
#include <QAction>
#include <QLineEdit>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QPushButton>
#include <QMessageBox>
#include <QLabel>
#include <QSvgRenderer>
#include <QPainter>
#include <QSvgWidget>
#include <QImage>
#include <QSvgWidget>
#include <QLabel>
#include <QScrollArea>
#include <QMouseEvent>
#include <QDir>
WeatherWidget::WeatherWidget(QWidget *parent, QString workpath) : QWidget(parent)
{
    // 设置天气数据回调函数
    setBackgroundRole(QPalette::Light);
    //设置窗口大小
    resize(700,400);
    setWindowFlag(Qt::FramelessWindowHint);
    wdl.queryCityWeather(currentCityName);
    QObject::connect(&wdl, &WeatherDataLoader::succeed, this, &WeatherWidget::onAddInformation);
    QObject::connect(&wdl, &WeatherDataLoader::failed, this, &WeatherWidget::failed);
    mouseMenu();
    move(300,300);
}
WeatherWidget::WeatherWidget(QSettings *m,QString workPath)
{
    this->workPath = workPath;
    setBackgroundRole(QPalette::Light);
    //设置窗口大小
    resize(700,400);
    setWindowFlag(Qt::FramelessWindowHint);
    wdl.queryCityWeather(currentCityName);
    QObject::connect(&wdl, &WeatherDataLoader::succeed, this, &WeatherWidget::onAddInformation);
    QObject::connect(&wdl, &WeatherDataLoader::failed, this, &WeatherWidget::failed);
    mouseMenu();
    int x = m->value("x").toInt();
    int y = m->value("y").toInt();
    this->move(x,y);
}
void WeatherWidget::setWorkPath(QString workPath)
{
    this->workPath = workPath;
}
QString WeatherWidget::getRand()
{
    int max = 8;
    QString tmp = QString("0123456789ABCDEFGHIJKLMNOPQRSTUVWXZY");
    QString str = QString();
    QTime t = QTime::currentTime();
    qsrand(t.msec()+t.second()*1000);
    for(int i = 0;i<max;i++){
        int ir = qrand()%tmp.length();
        str[i] = tmp.at(ir);
    }
    return str;
}
WeatherWidget::~WeatherWidget()
{
    if(isSave){
    QDir filePath{this->workPath};
    if(!filePath.exists("cache")){
      filePath.mkdir("cache");
    }
    filePath.cd("./cache");
    QString path =this->workPath+"/cache/"+getRand()+".ini";

    QSettings m(path,QSettings::IniFormat);
    m.setValue("x",this->x());
    m.setValue("y",this->y());
    }

}

void WeatherWidget::failed()
{

    int r = QMessageBox::warning(nullptr,"网络请求失败","天气请求失败，请检查网络是否连接",QMessageBox::Yes);
    if(r == QMessageBox::Yes){
        close();
        w = QMessageBox::Close;
    }

}

void WeatherWidget::onAddInformation(QVariant var_data)
{

    //设置主布局,分为地点，今日天气，未来天气
    QVBoxLayout *mainLayout = new QVBoxLayout;
    //获取首日天气数据
    this->currentCityName = wdl.getCityName();
    wdList =var_data.value<QList<WeatherData>>();
    auto wd = wdList[0];


    //IP地址的显示
    QFont ft;
    ft.setPointSize(24);
    QHBoxLayout *topLayout = new QHBoxLayout;
    QVBoxLayout *leftLayout = new QVBoxLayout;
    QLabel *location = new QLabel(currentCityName);
    location->setFont(ft);

    leftLayout->addWidget(location);
    //编写今日天气的显示
    QHBoxLayout *todayLayout = new QHBoxLayout;
    QLabel *todayTem = new QLabel(wd.degree+"°C");//气温
    todayTem->setFont(ft);
    todayLayout->addWidget(todayTem);
    QLabel *todayData = new QLabel(wd.date);//日期
    todayData->setFont(ft);
    todayLayout->addWidget(todayData);
    QVBoxLayout *windyandtypeLayout = new QVBoxLayout;
    QLabel *todayWindy = new QLabel("风力:"+wd.windy);
    QLabel *todayType = new QLabel("天气:"+wd.type);
    windyandtypeLayout->addWidget(todayWindy);
    windyandtypeLayout->addWidget(todayType);
    todayLayout->addLayout(windyandtypeLayout);

    leftLayout->addLayout(todayLayout);
    topLayout->addLayout(leftLayout);
    QSvgWidget *icon =new QSvgWidget(QString(":/icons/%1.svg").arg(wd.icon));//天气图标
    icon->resize(300,300);
    topLayout->addWidget(icon);

    mainLayout->addLayout(topLayout);




    //编写往后几天的天气显示
    QScrollArea *bottomLayout = new QScrollArea;
    QWidget *w = new QWidget;
    w->resize(1000,250);
    bottomLayout->setWidget(w);
    QHBoxLayout *allDayLayout = new QHBoxLayout;
    w->setLayout(allDayLayout);
    for(int i = 1;i< 10;i++){
        wd = wdList[i];
        QVBoxLayout *oneDayLayout = new QVBoxLayout;
        QLabel *oneDate = new QLabel(wd.date);
        oneDate->setMaximumSize(100,25);
        oneDate->setMinimumSize(100,25);
        oneDayLayout->addWidget(oneDate);
        QSvgWidget *oneIcon =new QSvgWidget(QString(":/icons/%1.svg").arg(wd.icon));//天气图标
        oneIcon->resize(100,250);
        oneDayLayout->addWidget(oneIcon);
        QLabel *oneType = new QLabel(wd.type);
        oneType->setMaximumSize(100,25);
        oneType->setMinimumSize(100,25);
        oneDayLayout->addWidget(oneType);
        QLabel *oneTmp = new QLabel(wd.degree+"°C");//气温
        oneTmp->setMaximumSize(100,25);
        oneTmp->setMinimumSize(100,25);
        oneDayLayout->addWidget(oneTmp);
        allDayLayout->addLayout(oneDayLayout);
    }
    mainLayout->addWidget(bottomLayout);
    setLayout(mainLayout);
}

void WeatherWidget::mousePressEvent(QMouseEvent *e)
{
    if(e->button()==Qt::LeftButton){
        moveIf = true;
        clickPos = e->pos();
    }
    return QWidget::mousePressEvent(e);
}

void WeatherWidget::mouseMoveEvent(QMouseEvent *e)
{
    if(moveIf)move(e->globalPos()-clickPos);
    return QWidget::mouseMoveEvent(e);
}

void WeatherWidget::mouseReleaseEvent(QMouseEvent *e)
{
    moveIf =false;
    return QWidget::mouseReleaseEvent(e);
}


void WeatherWidget::mouseMenu()
{
    QAction *closeAction = new QAction("关闭");
    addAction(closeAction);
    connect(closeAction,&QAction::triggered,this,[=](){
        isSave = true;
        this->close();
    });
    setContextMenuPolicy(Qt::ActionsContextMenu);
}

