# 项目中期报告

## 一、背景介绍

- 小组名称：电脑配舰队
- 赛题：[proj175-基于uos的DDE核心组件实现桌面小部件](https://github.com/oscomp/proj175-desktop-gadgets-in-uos)
- 项目简介：本次项目要求为使用QT插件化技术开发天气、便签、时钟桌面插件。

## 二、技术方案

### 1、QT插件介绍

插件是一种（遵循一定规范的应用程序接口编写出来的）程序，定位于开发实现应用软件平台不具备的功能的程序。插件必须依赖于应用程序才能发挥自身功能，仅靠插件是无法正常运行的；
相反地，应用程序并不需要依赖插件就可以运行，这样一来，插件就可以加载到应用程序上并且动态更新而不会对应用诚信度造成任何改变（热更新）。
Qt提供了两种API用于创建插件：

- 一种是高阶API，用于扩展Qt本身的功能：例如自定义数据库驱动、图像格式、文本编码、自定义样式等等；
- 一种是低阶API，用于扩展Qt应用程序；

即我们可以使用静态和动态两种方式来调用插件，基于本次项目的要求，我们选择了动态插件开发的方法进行项目功能实现。
 
### 2、项目插件开发流程

#### 2.1、 插件结构

Qt的插件开发至少分为两部分：插件管理器部分和插件部分
插件管理器部分：定义插件的接口并提供插件的管理器用于管理插件的加载与使用；
插件程序部分：用于按照主程序中定义的插件接口来定义插件，最终实现插件的功能，并生成供主程序部分调用的插件；

项目中插件管理器的项目为`PluginDemo`，里面定义了所有插件必须实现的接口函数，对插件的行为进行约束并管理插件的生命周期。插件部分对应3个子项目，分别是`clock`、`memo`、`weather`，即时钟、便签、天气插件，这些项目实现插件管理器的接口定义的实现和插件功能的实现。整体结构分为插件实现和功能实现2个部分。

- 插件管理器接口定义步骤

1. 定义一组用于与插件通信的接口（只有纯虚函数的类）；
2. 使用Q_DECLARE_INTERFACE()宏来告诉Qt源对象系统有关接口的情况；
3. 在应用程序中使用QPluginLoader加载插件；

本项目中插件管理器接口定义如下所示：

```
class CalInterface
{
public:
    virtual ~CalInterface() {}
    virtual QWidget* getWidget(QString workPath) = 0; // 获取插件Widget
    virtual QList<QWidget*> loadAll(QString workPath) = 0; // 加载离线插件
};

#define CalInterface_iid "Examples.Plugin.CalInterface"

QT_BEGIN_NAMESPACE
Q_DECLARE_INTERFACE(CalInterface,CalInterface_iid)
QT_END_NAMESPACE
```

接口中主要定义了2个功能：

- getWidget方法

 该方法用于第一次加载插件到插件管理器中，并将插件实例化显示。

- loadAll方法

 插件在运行过程中会产生很多中间数据，因此我们在插件中使用QT提供的数据缓存接口技术实现了插件数据持久化和自启动加载功能。`loadAll`方法主要用于加载进行数据持久化之后的插件。

为了展示上述2个接口的功能，下面展示了时钟插件中接口的实现代码:

```
QWidget* ClockPlugin::getWidget(QString workPath)
{
    clockwidget *newWindow = new clockwidget;	// 初始化时钟窗口
    newWindow->setWorkPath(workPath);		// 设置时钟运行目录
    return newWindow;				// 时钟实例化对象返回给插件管理器
}
```

```
QList<QWidget*> ClockPlugin::loadAll(QString workPath)
{
    QList<QWidget*> listClock;
    QDir dirName{workPath};	
    if(dirName.exists("cache")){
        dirName.cd("./cache");			// 打开插件数据持久化目录
        dirName.setFilter(QDir::NoDotAndDotDot|QDir::AllEntries);
        QStringList fileNames = dirName.entryList();//遍历目录下的所有条目
        for(auto file:fileNames){
            if(file.endsWith(".ini")){		// 匹配配置文件
                QSettings *loadFile = new QSettings(workPath+"/cache/"+file,QSettings::IniFormat); // 读取插件中间数据
                clockwidget *weatherWindow = new clockwidget(loadFile,workPath); // 初始化时钟窗口
                listClock.push_back(weatherWindow);	// 加入实例化对象列表
                dirName.remove(file);		// 移除配置文件
            }
        }
        return listClock;
    }
    return QList<QWidget*>();
}
```


### 2.2、第3方插件开发步骤

#### 2.2.1、定义插件接口

按照上述的插件管理器接口定义，第3方插件首先要实现getWidget和loadAll方法。
下面是上述2个方法的功能描述，具体实现请参考上述时钟代码。

```
QWidget* ClockPlugin::getWidget(QString workPath)
{
    1. 初始化插件启动窗口实例
    2. 设置插件运行目录，该目录是插件管理器的工作目录，方便统一管理插件，保持中间数据
    3. 将插件实例化对象返回给插件管理器
}
```

```
QList<QWidget*> ClockPlugin::loadAll(QString workPath)
{
   1. 遍历插件缓存目录（插件运行后的中间数据保存在插件安装目录下的cache目录中）
   2. 读取插件配置文件
   3. 解析配置文件，调用插件中含有设置配置参数的构造函数初始化插件实例(cache目录中每个缓存文件对应一个插件实例)
   4. 将插件实例化对象列表返回给插件管理器
}
```

#### 2.2.2、打包插件

在PluginManager的plugins目录下有3个已经打包好的插件。一个典型的插件打包目录如下所示：

![](./imgs/plugin_struct.png)

每个文件的功能描述如下：

|  文件名   | 功能  |
|  ----  | ----  |
| libclockplugin.so  | 插件生成的动态库 |
| clock.webp  | 插件图标(不限于webp格式，常用图片格式都可) |
| config.ini  | 插件配置文件，用于插件管理器加载插件，该文件格式由插件管理器统一定义，不可随意修改！ |

`config.ini`文件中定义了插件管理器加载插件时依赖的数据，文件内容如下所示：

```
[General]
icon=clock.webp	
name=libclockplugin.so
version=1.0
plugName=时钟
```

其中
`icon` : 指定插件展示图标
`name` : 指定插件运行库名称
`version` : 指定插件版本号
`plugName` : 指定插件在插件管理器中的展示名称

在插件打包过程中，为了避免插件配置错误的情况，强烈建议直接复制已有的插件打包目录再替换内容。
插件目录配置完成后，将目录手动复制到插件管理器运行目录的`plugins`中，即可完成插件的安装。
